(function() {
	if (window.WebViewJavascriptBridge) { return }
	var receiveMessageQueue = []
	var messageHandlers = {}
	
	var CUSTOM_PROTOCOL_SCHEME = 'wvjbscheme'
	var QUEUE_HAS_MESSAGE = '__WVJB_QUEUE_MESSAGE__'
	
	var responseCallbacks = {}
	var uniqueId = 1

	function init(messageHandler) {
		if (WebViewJavascriptBridge._messageHandler) { throw new Error('WebViewJavascriptBridge.init called twice') }
		WebViewJavascriptBridge._messageHandler = messageHandler
		var receivedMessages = receiveMessageQueue
		receiveMessageQueue = null
		for (var i=0; i<receivedMessages.length; i++) {
			_dispatchMessageFromAndroid(receivedMessages[i])
		}
	}

	function send(data, responseCallback) {
		_doSend({ data:data }, responseCallback)
	}
	
	function registerHandler(handlerName, handler) {
		messageHandlers[handlerName] = handler
	}


	function callHandler(handlerName, data, responseCallback) {
        if(typeof(data) == 'function'){
            var responseCallback = data;
            _doSend({ handlerName:handlerName}, responseCallback);
        }else{
		    _doSend({ handlerName:handlerName, data:data }, responseCallback);
		}
	}

	function _doSend(message, responseCallback) {
		if (responseCallback) {
			var callbackId = 'cb_'+(uniqueId++)+'_'+new Date().getTime()
			responseCallbacks[callbackId] = responseCallback
			message['callbackId'] = callbackId
		}

		var messageString = JSON.stringify(message)
		NativeJavascriptBridge.handleMessage(messageString)
	}

	function _dispatchMessageFromAndroid(messageJSON) {
		setTimeout(function _timeoutDispatchMessageFromObjC() {
			var message = JSON.parse(messageJSON)
			var messageHandler
			var responseCallback

			if (message.responseId) {
				responseCallback = responseCallbacks[message.responseId]
				if (!responseCallback) { return; }
				responseCallback(message.responseData)
				delete responseCallbacks[message.responseId]
			} else {
				if (message.callbackId) {
					var callbackResponseId = message.callbackId
					responseCallback = function(responseData) {
						_doSend({ responseId:callbackResponseId, responseData:responseData })
					}
				}

				var handler = WebViewJavascriptBridge._messageHandler
				if (message.handlerName) {
					handler = messageHandlers[message.handlerName]
				}

				try {
					handler(message.data, responseCallback)
				} catch(exception) {
					if (typeof console != 'undefined') {
						console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception)
					}
				}
			}
		})
	}

	function _handleMessageFromAndroid(messageJSON) {
    	if (receiveMessageQueue) {
    		receiveMessageQueue.push(messageJSON)
    	} else {
    		_dispatchMessageFromAndroid(messageJSON)
   		}
   	}

	window.WebViewJavascriptBridge = {
		init: init,
		send: send,
		registerHandler: registerHandler,
		callHandler: callHandler,
		_handleMessageFromAndroid: _handleMessageFromAndroid
	}

	var doc = document
	var readyEvent = doc.createEvent('Events')
	readyEvent.initEvent('WebViewJavascriptBridgeReady')
	readyEvent.bridge = WebViewJavascriptBridge
	doc.dispatchEvent(readyEvent)
	NativeJavascriptBridge.showLog('WebViewJavascriptBridgeReady')

//	function connectWebViewJavascriptBridge(callback) {
//        if (window.WebViewJavascriptBridge) {
//            callback(WebViewJavascriptBridge)
//        } else {
//            document.addEventListener('WebViewJavascriptBridgeReady', function() {
//                callback(WebViewJavascriptBridge)
//            }, false)
//        }
//    }
//
//    connectWebViewJavascriptBridge(function(bridge) {
//        console.log("callbridge functions 1")
//
//        bridge.init(function(message, responseCallback) {
//            NativeJavascriptBridge.showLog('Received message: ' + message)
//            if (responseCallback) {
//                responseCallback("Right back atcha")
//            }
//        })
//        var shareContent = new Object()
//        shareContent.title = 'test title'
//        shareContent.content = 'test content'
//        shareContent.url = 'testUrl'
//        bridge.callHandler('Social.Share', shareContent, function(result){
//            console.log(result['status'])
//        })
//    })
}())

